/*
** exutils.h
**
** Header file which contains the defines and prototypes for the utility
** functions in exutils.c and exutils2.c.
** 
*/

/*****************************************************************************
** 
** defines and typedefs used 
** 
*****************************************************************************/

#ifndef MAX
#define MAX(X,Y)	(((X) > (Y)) ? (X) : (Y))
#endif

#ifndef MIN
#define MIN(X,Y)	(((X) < (Y)) ? (X) : (Y))
#endif

/*
** Maximum character buffer for displaying a column
*/
#define MAX_CHAR_BUF	1024

/*
** Define structure where row data is bound.
*/
typedef struct _ex_column_data
{
	CS_INT		indicator;
	CS_CHAR		*value;
	CS_INT		valuelen;
} EX_COLUMN_DATA;

typedef struct _ct_scroll_indexlist
{
        int          index;
        int          scrolltype;
} CT_SCROLL_INDEX;

/*
** Data structure for array binding of scrollable cursor
** results. This is an illustrative example only, there
** are different and more flexible ways to do columns.
** Column length is fixed in this example. It is advised
** to set ARRAY_BND_LEN to CS_CURSOR_ROWS for efficiency.
*/
#define ARRAY_BND_LEN 	5	/* so set CS_CURSOR_ROWS to 5 */
#define	MAX_COLUMN_LEN	45	/* maximum size of a column */

typedef struct ex_column_array {
	CS_DATAFMT	datafmt;
	CS_CHAR		value[ARRAY_BND_LEN][MAX_COLUMN_LEN];
	CS_INT		valuelen[ARRAY_BND_LEN];
	CS_SMALLINT	indicator[ARRAY_BND_LEN];
} COLUMN_ARRAY; 

/*****************************************************************************
** 
** protoypes for all public functions 
** 
*****************************************************************************/
/* exutils.c */
extern CS_INT CS_PUBLIC ex_display_dlen PROTOTYPE((
	CS_DATAFMT *column
	));
extern CS_RETCODE CS_PUBLIC ex_display_header PROTOTYPE((
	CS_INT numcols,
	CS_DATAFMT columns[]
	));
extern CS_RETCODE CS_PUBLIC ex_display_column PROTOTYPE((
	CS_CONTEXT *context,
	CS_DATAFMT *colfmt,
	CS_VOID *data,
	CS_INT datalength,
	CS_INT indicator
	));
extern CS_VOID CS_PUBLIC ex_panic PROTOTYPE((
	char *msg
	));
extern CS_VOID CS_PUBLIC ex_error PROTOTYPE((
	char *msg
	));
extern CS_RETCODE CS_PUBLIC ex_clientmsg_cb PROTOTYPE((
	CS_CONTEXT *context,
	CS_CONNECTION *connection,
	CS_CLIENTMSG *errmsg
	));
extern CS_RETCODE CS_PUBLIC ex_servermsg_cb PROTOTYPE((
	CS_CONTEXT *context,
	CS_CONNECTION *connection,
	CS_SERVERMSG *srvmsg
	));
extern CS_RETCODE CS_PUBLIC ex_init PROTOTYPE((
	CS_CONTEXT **context
	));
extern CS_RETCODE CS_PUBLIC ex_connect PROTOTYPE((
	CS_CONTEXT *context,
	CS_CONNECTION **connection,
	CS_CHAR *appname,
	CS_CHAR *username,
	CS_CHAR *password,
	CS_CHAR *server
	));
extern CS_RETCODE CS_PUBLIC ex_con_cleanup PROTOTYPE((
	CS_CONNECTION *connection,
	CS_RETCODE status
	));
extern CS_RETCODE CS_PUBLIC ex_ctx_cleanup PROTOTYPE((
	CS_CONTEXT *context,
	CS_RETCODE status
	));
extern CS_RETCODE CS_PUBLIC ex_execute_cmd PROTOTYPE((
	CS_CONNECTION *connection,
	CS_CHAR *cmdbuf
	));
extern CS_RETCODE CS_PUBLIC ex_fetch_data PROTOTYPE((
	CS_COMMAND *cmd
	));
extern CS_RETCODE CS_PUBLIC ex_create_db PROTOTYPE((
	CS_CONNECTION *connection,
	char *dbname
	));
extern CS_RETCODE ex_remove_db PROTOTYPE((
	CS_CONNECTION *connection,
	char *dbname
	));
extern CS_RETCODE ex_use_db PROTOTYPE((
	CS_CONNECTION *connection,
	char *dbname
	));
